package com.gic.store.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.log.api.service.LogApiService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.CreateTypeEnum;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.constant.StoreLogReasonEnum;
import com.gic.store.dto.*;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreAuthorizationApiService;
import com.gic.store.service.StoreDictApiService;
import com.gic.store.service.StoreLogApiService;
import com.gic.store.utils.StoreRedisKeyUtils;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.store.StoreQO;
import com.gic.store.web.vo.StoreListSourceVO;
import com.gic.store.web.vo.StoreOverflowVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * @author zhiwj
 * @date 2019/6/28
 */
@RestController
public class StoreController {

    private static final Logger logger = LogManager.getLogger(StoreController.class);

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreDictApiService storeDictApiService;
    @Autowired
    private LogApiService logApiService;
    @Autowired
    private StoreLogApiService storeLogApiService;

    @RequestMapping("/save-store")
    public RestResponse saveOrUpdate(@Validated({StoreQO.SaveValidView.class}) StoreQO storeQO) {

        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        StoreDTO storeDTO = EntityUtil.changeEntityByOrika(StoreDTO.class, storeQO);
        storeDTO.setEnterpriseId(enterpriseId);
        String photos = storeQO.getPhotos();
        if (StringUtils.isNotBlank(photos)) {
            List<StorePhotoDTO> photoList = JSON.parseArray(photos, StorePhotoDTO.class);
            storeDTO.setPhotoList(photoList);
        }
        String businessTimes = storeQO.getBusinessTimes();
        if (StringUtils.isNotBlank(businessTimes)) {
            List<StoreBusinessTimeDTO> businessTimeList = JSON.parseArray(businessTimes, StoreBusinessTimeDTO.class);
            storeDTO.setBusinessTimeList(businessTimeList);
        }
        storeDTO.setCreateType(CreateTypeEnum.BACKGROUND_ADD.getCode());
        String fieldJson = storeQO.getStoreExtends();
        List<StoreExtendDTO> extendList = JSON.parseArray(fieldJson, StoreExtendDTO.class);
        storeDTO.setStoreExtendList(extendList);
        // 判断是否手动修改过分组
        Integer isEditGroup = getIsEditGroup(enterpriseId, storeDTO);
        logger.info("isEditGroup:{}", isEditGroup);
        storeDTO.setIsEditStoreGroup(isEditGroup);
        storeDTO.setReason(StoreLogReasonEnum.WEB.getCode());
        storeDTO.setRemark(storeQO.getStoreId() == null ? "手工新增" : "单个编辑");
        storeDTO.setOperatorId(UserDetailUtils.getUserDetail().getUserId());
        storeDTO.setOperatorName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        ServiceResponse<Integer> serviceResponse = storeApiService.saveOrUpdate(storeDTO);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog((storeQO.getStoreId() == null ? "新增" : "修改") + "门店", storeQO.getStoreName());
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/store-detail")
    public RestResponse getStoreDetail(Integer storeId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<StoreDTO> serviceResponse = storeApiService.getStoreById(enterpriseId, storeId);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/store-list")
    public RestResponse storeList(@RequestBody StoreSearchDTO storeSearchDTO) {
        logger.info("查询条件：{}", JSON.toJSONString(storeSearchDTO));
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        String key = StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId);
        Object obj = RedisUtil.getCache(key);
        String returnFileds = StoreESFieldsEnum.STOREID.getField();
        if(obj == null){
            List<StoreListSourceVO> list = this.getStoreAllListSource(enterpriseId);
            for(StoreListSourceVO vo : list){
                if(vo.getChecked().intValue() == 1){
                    returnFileds += vo.getValue()+",";
                }
            }
        }else {
            returnFileds += ","+obj.toString();
        }
        ServiceResponse<Page<StoreDTO>> serviceResponse = storeApiService.listStore(storeSearchDTO, storeSearchDTO.getCurrentPage(), storeSearchDTO.getPageSize(), returnFileds);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("store-list-source")
    public RestResponse storeListSource() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        List<StoreListSourceVO> list = this.getStoreAllListSource(enterpriseId);
        Object obj = RedisUtil.getCache(StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId));
        if (obj == null) {
            return RestResponse.success(list);
        }
        String[] sourcesArr = obj.toString().split(",");
        for (StoreListSourceVO storeListSourceVO : list) {
            boolean flag = false;
            for (String s : sourcesArr) {
                if (s.equals(storeListSourceVO.getValue())) {
                    storeListSourceVO.setChecked(1);
                    flag = true;
                }
            }
            if(!flag){
                storeListSourceVO.setChecked(0);
            }
        }
        return RestResponse.success(list);
    }

    @RequestMapping("store-list-source-set")
    public RestResponse storeListSourceSet(String sources) {
        if (StringUtils.isBlank(sources)) {
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        RedisUtil.setCache(StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId), sources);
        logger.info("自定义门店列表字段配置:{}", JSON.toJSONString(RedisUtil.getCache(StoreRedisKeyUtils.getStoreListSourceKey(enterpriseId, userId))));
        return RestResponse.success();
    }

    @RequestMapping("list-store-overflow")
    public RestResponse listStoreOverFlow(String search, PageQO pageQO) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Page<StoreDTO> page = new Page<>(pageQO.getCurrentPage(), pageQO.getPageSize());
        ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStoreOverFlow(enterpriseId, page, search);
        if (response.isSuccess()) {
            page = response.getResult();
            Page<StoreOverflowVO> page1 = new Page<>();
            page1.setTotalCount(page.getTotalCount());
            page1.setCurrentPage(page.getCurrentPage());
            page1.setPageSize(page.getPageSize());
            page1.setResult(EntityUtil.changeEntityListByJSON(StoreOverflowVO.class, page.getResult()));
            return RestResponse.success(page1);
        }
        return EnterpriseRestResponse.failure(response);
    }

//    @RequestMapping("list-store-log")
//    public RestResponse listStoreLog(Integer storeId, PageQO pageQO) {
//        Page page = new Page(pageQO.getCurrentPage(), pageQO.getPageSize());
//        Map<String, Object> params = new HashMap<>();
//        params.put("relationId", storeId);
//        page.setParams(params);
//        ServiceResponse response = this.logApiService.listSystemSetLog(page);
//        if (response.isSuccess()) {
//            return RestResponse.success(response.getResult());
//        }
//        return EnterpriseRestResponse.failure(response);
//    }
    @RequestMapping("list-store-log")
    public RestResponse listStoreLog(com.gic.enterprise.qo.PageQO pageQO, String content, Integer reason, Integer logType, Integer storeId) {
        StoreLogDTO storeLog = new StoreLogDTO();
        storeLog.setContent(content);
        storeLog.setReason(reason);
        storeLog.setLogType(logType);
        storeLog.setStoreId(storeId);
        ServiceResponse<Page<StoreLogDTO>> response = storeLogApiService.listStoreLog(storeLog, pageQO);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("bulk-update-store")
    public RestResponse bulkUpdateStatus(String storeIds, String value, int type) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (StringUtils.isBlank(storeIds) || StringUtils.isBlank(value)) {
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        ServiceResponse response = this.storeApiService.bulkUpdateStore(storeIds, value, enterpriseId, type, UserDetailUtils.getUserDetail().getUserId(), UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        if (response.isSuccess()) {
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }


    private List<StoreListSourceVO> getStoreAllListSource(Integer enterpriseId) {
        List<StoreListSourceVO> list = new ArrayList<>();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listStoreField(enterpriseId);
        if (serviceResponse.isSuccess()) {
            List<StoreDictDTO> storeDictDTOList = serviceResponse.getResult();
            if (CollectionUtils.isNotEmpty(storeDictDTOList)) {
                for (StoreDictDTO storeDictDTO : storeDictDTOList) {
                    if (storeDictDTO.getChecked() == 1) {
                        StoreListSourceVO vo = new StoreListSourceVO();
                        vo.setChecked(0);
                        vo.setIsCustom(1);
                        vo.setKey(storeDictDTO.getKey());
                        String key = StoreRedisKeyUtils.getStoreFieldKey(enterpriseId, storeDictDTO.getStoreDictId());
                        Object object = RedisUtil.getCache(key);
                        if(object != null){
                            vo.setValue(object.toString());
                            list.add(vo);
                        }
                    }
                }
            }
        }
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STORENAME.getField(), StoreESFieldsEnum.STORENAME.getDesc(), 1));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STOREGROUPNAME.getField(), StoreESFieldsEnum.STOREGROUPNAME.getDesc(), 1));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STORECODE.getField(), StoreESFieldsEnum.STORECODE.getDesc(), 1));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.PROVINCES.getField(), StoreESFieldsEnum.PROVINCES.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STOREBRANDS.getField(), StoreESFieldsEnum.STOREBRANDS.getDesc() ));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.COMPLETESTATUS.getField(), StoreESFieldsEnum.COMPLETESTATUS.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STATUS.getField(), StoreESFieldsEnum.STATUS.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.REGIONNAME.getField(), StoreESFieldsEnum.REGIONNAME.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.ERPSTATUS.getField(), StoreESFieldsEnum.ERPSTATUS.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STORETYPE.getField(), StoreESFieldsEnum.STORETYPE.getDesc()));
        list.add(new StoreListSourceVO(StoreESFieldsEnum.STOREID.getField(), StoreESFieldsEnum.STOREID.getDesc(),1));
        return list;
    }

    private Integer getIsEditGroup(Integer enterpriseId, StoreDTO storeDTO) {
        if (storeDTO.getIsEditStoreGroup() != null && storeDTO.getIsEditStoreGroup() == 1) {
            return 1;
        }
        Integer isEditGroup;
        Integer newGroupId = storeDTO.getStoreGroupId();
        if (storeDTO.getStoreId() == null) {
            isEditGroup = newGroupId == null ? 0 : 1;
        } else {
            StoreDTO oldDTO = storeApiService.getStoreById(enterpriseId, storeDTO.getStoreId()).getResult();
            if (oldDTO == null) {
                return 0;
            }
            Integer oldGroupId = oldDTO.getStoreGroupId();
            if (newGroupId == null && oldGroupId == null) {
                isEditGroup = 0;
            } else if (oldGroupId != null) {
                isEditGroup = oldGroupId.equals(newGroupId) ? 0 : 1;
            } else {
                isEditGroup = newGroupId.equals(oldGroupId) ? 0 : 1;
            }
        }
        return isEditGroup;
    }
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;

    @RequestMapping("test-a")
    @ResponseBody
    public Object test(String params){
        ServiceResponse<Void> voidServiceResponse = this.storeAuthorizationApiService.authorizationSingleStore(params);
        return voidServiceResponse;
    }

}